#!/bin/bash
SCRIPTPATH="$( cd -- "$(dirname "$0")" >/dev/null 2>&1 ; pwd -P )"

mkdir -p /var/lib/iqnergy/

NODE_TYPE=
LOGIN=
PASSWORD=
NODE_ID=
HANDLE_ERROR=
BRIDGE_CFG=/mnt/data/etc/mosquitto/conf.d/20bridge.conf
LISTENERS_CFG=/mnt/data/etc/mosquitto/conf.d/10listeners.conf
INITIAL_CFG=/etc/mosquitto/mosquitto.conf

help () {
    echo "To install the controller, set the parameters:"
    echo "-t, --type            TYPE       [ dev | stage | online ]"
    echo "-i, --id              NODE_ID    Some controller id without spaces."
    echo "-u, --mqtt_username   USERNAME   Username on MQTT Server."
    echo "-p, --mqtt_password   PASSWORD   Password on MQTT Server."
    echo "-a, --add                        Add new, no remove previous."
    echo
    echo "--handle_error                   Move to the stage in case of an error."
}

find_environment() {
    TIMESTAMP=$(date +"%s")

    USERNAME_LINE=$(grep -a -E ^remote_username ${1} | head -n 1)
    PASSWORD_LINE=$(grep -a -E ^remote_password ${1} | head -n 1)
    NODE_ID_LINE=$(grep -a -E ^connection ${1} | head -n 1)
    SERVER_LINE=$(grep -a -E ^address ${1} | head -n 1)

    SERVER_LINE_S1=${SERVER_LINE#* }
    NODE_ID_S1=${NODE_ID_LINE#*_}

    NODE_TYPE=${SERVER_LINE_S1%.iqnergy*}
    LOGIN=${USERNAME_LINE#* }
    PASSWORD=${PASSWORD_LINE#* }
    NODE_ID=${NODE_ID_S1%_*}
}

# Ищем данные входа в bridge.cfg
find_environment ${BRIDGE_CFG}

if  [ -z "$NODE_TYPE" ] || \
    [ -z "$NODE_ID" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ];
then
    # Ищем данные входа в mosquitto.cfg
    find_environment ${INITIAL_CFG}
fi


while [ -n "$1" ]
do
case "$1" in
-t|--type)
    NODE_TYPE="$2"
    shift ;;

-i|--id)
    NODE_ID="$2"
    shift ;;

-u|--mqtt_username)
    LOGIN="$2"
    shift ;;

-p|--mqtt_password)
    PASSWORD=$2
    shift ;;

--upgrade)
    shift ;;

--handle_error)
    HANDLE_ERROR=true ;;

-a|--add)
    ADD_BRIDGE=true ;;

*) help ;;
esac
shift
done


if  [ -z "$NODE_TYPE" ] || \
    [ -z "$NODE_ID" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ];
then
    help;
    exit 1;
fi

# NODE_ID == wirenboard только когда собираем OS в Docker
# В таком случае ID менять смысла нет
if [[ $NODE_ID == "wirenboard" ]]
then
    NODE_ID=$(cat /etc/hostname)
fi

if [ -z "$NODE_TYPE" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ]; then

    # если контроллер потерялся, перенаправляем его на stage
    if [ ! -z "$HANDLE_ERROR" ]; then
        NODE_TYPE=stage
        LOGIN=iqdata
        PASSWORD='NC^R5rfubI7cGF'
        NODE_ID="LOST_${NODE_ID}"
    fi
fi

echo "**************************"
echo "Found bridge configuration:"
echo $NODE_TYPE
echo $LOGIN
echo $NODE_ID
echo "**************************"
echo "**************************"

CA_CERTIFICATES="/etc/mosquitto/ca_certificates"
mkdir -p ${CA_CERTIFICATES}

declare -A IQ_CERTIFICATES

IQ_CERTIFICATES["mqtt-dev"]='
-----BEGIN CERTIFICATE-----
MIID0TCCArmgAwIBAgIUate/xqpux7Xnkrvl/a8rYUCa/SowDQYJKoZIhvcNAQEL
BQAweDELMAkGA1UEBhMCUlUxDzANBgNVBAgMBk1vc2NvdzEPMA0GA1UEBwwGTW9z
Y293MRAwDgYDVQQKDAdJUW5lcmd5MRQwEgYDVQQDDAtpcW5lcmd5LmNvbTEfMB0G
CSqGSIb3DQEJARYQaW5mb0BpcW5lcmd5LmNvbTAeFw0yNDA0MTAxNjU0MDhaFw0z
NDA0MDgxNjU0MDhaMHgxCzAJBgNVBAYTAlJVMQ8wDQYDVQQIDAZNb3Njb3cxDzAN
BgNVBAcMBk1vc2NvdzEQMA4GA1UECgwHSVFuZXJneTEUMBIGA1UEAwwLaXFuZXJn
eS5jb20xHzAdBgkqhkiG9w0BCQEWEGluZm9AaXFuZXJneS5jb20wggEiMA0GCSqG
SIb3DQEBAQUAA4IBDwAwggEKAoIBAQDd/N4kc4cIJLoCrxbtTXfCmChhxyGtOmWf
h5owB3GJ08byITp9I5Kg/lvKExXI2OPaGWOM+pM8IbMo0zXd9SRXMYhwm345UAro
LNYI3to7+4e6ySzd/E+W2g79WpF2AwAxX/3saBnG52Fv7jWgIe/eLQerY+2CIa+y
t65zGaFZQSd/RTtOMOse69o6sqbvkXvoNnPKywac5g+o3YDdlVW7tNVzQEiUuVjM
DTw2NzcU+yToK2McO6MEkZcRR7JHvkooyDIf8N4tvirwK3ByhTo2lk6YuSIp1Ces
T3oSH/4j8mQlqviTWkpnxeQ4lZ6bd8i9/vEtOBipr085K/MITwWrAgMBAAGjUzBR
MB0GA1UdDgQWBBSs38uono2abf9DQ+APWb4i48lRDjAfBgNVHSMEGDAWgBSs38uo
no2abf9DQ+APWb4i48lRDjAPBgNVHRMBAf8EBTADAQH/MA0GCSqGSIb3DQEBCwUA
A4IBAQDYnZ4NQB0LZgVb81xhhazYh7wQ3JrtRQXjWENGTgr0/MOTuv9y6Rn0LB2J
lBIlBgK8FX1VuWGM1ohn17ym/rNTvumMqTbe+Gafhxfb0IL/sB0Vs6eJCq6en2za
0SExMc3h+LEqWRDsAKDAGBZ5TSVloca5EEh7+meGktJJQVLTJUMRx2cot3BzhsWV
LvwikS0S2Sn3l18FbiHEDATTgsRchH9fB4sDUokdKzwcAXQ2lTP4rge//JA8Jflp
PUqVBsGLK+jxvvlJypCapLTwm56h6inCL6XxEa8SHSOA/5kSuPqnm85JpTWhtNSm
vlozj/jGMu2COb6zSH55HGbooxhW
-----END CERTIFICATE-----
'
IQ_CERTIFICATES["stage"]='
-----BEGIN CERTIFICATE-----
MIIDgTCCAmmgAwIBAgIJAMhVMeEYQRU3MA0GCSqGSIb3DQEBCwUAMFcxCzAJBgNV
BAYTAlJVMQ8wDQYDVQQIDAZNb3Njb3cxDzANBgNVBAcMBk1vc2NvdzEQMA4GA1UE
CgwHSVFuZXJneTEUMBIGA1UEAwwLaXFuZXJneS5jb20wHhcNMTkwNTEzMTAwMTUz
WhcNMjkwNTEwMTAwMTUzWjBXMQswCQYDVQQGEwJSVTEPMA0GA1UECAwGTW9zY293
MQ8wDQYDVQQHDAZNb3Njb3cxEDAOBgNVBAoMB0lRbmVyZ3kxFDASBgNVBAMMC2lx
bmVyZ3kuY29tMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA0rO7GMg5
7rjEAsl3QjbbpCpHTw99qqSOLB6dmbhrYrpCx20sz2oJ+qgo+ekQyiOLKZ4LfzTv
jjRzmjPJtTycap/FTaknMQHK1oqoEzbjlyr+JMFiP/Jp11RSSy17737WjJaaMfoF
kRMIViYJmFVevK7w9VUZOUozJ/pFPqcLdxfjjpeDLZaCpDI36l3D7q1gtFe/jykg
6gF7qLVo7fmh8clyijst3zSWcqEoZt2GTW1zb2QkgOnKp/JWktnJzd0mFLujw1Bt
cGUdXX94/fOnd1umGSOqJtp/4VStzg12LRUO7IaPfW0ahh4XFmn83T2ITP9wb0Lw
prA3hZm/YafxAwIDAQABo1AwTjAdBgNVHQ4EFgQUHmmjk2dRF9/MQn5oE2EAXG94
OewwHwYDVR0jBBgwFoAUHmmjk2dRF9/MQn5oE2EAXG94OewwDAYDVR0TBAUwAwEB
/zANBgkqhkiG9w0BAQsFAAOCAQEAhM0eIdKC63o21GQd65JIzMk2sT7jVyzD/bcp
QPbOYOaWV/nQwyX7bZvR7yL/ONpFeXyrzs1EgTfijd0PQ8eY3O22Ve09LUjXEYjW
h/IfBjJinLZ9CD2oR1Q8pgKLUBA5R6O4v83t+4OVIubyi0Yx4aqNKZuuXFjOBVYI
BwOze237Wrrr6hcKjRS2Z0rJZrWOuoNxECoOkLPF+1N5sfc3QvmLavWKE1/Klmd2
UK/i9ZjoGaKV3ewXdR4CsiCsaHy3PQLmEtsAZq6UGS8B0rDdSBunsfp9oqrmM43A
m6yWsXIGZTd2oaVbiIVZ9Y8/PtiuFDWczjRTuGZ6yQ1ccNwmbg==
-----END CERTIFICATE-----
'

IQ_CERTIFICATES["online"]='
-----BEGIN CERTIFICATE-----
MIIDgTCCAmmgAwIBAgIJAJv2Q5wwjWJxMA0GCSqGSIb3DQEBCwUAMFcxCzAJBgNV
BAYTAlJVMQ8wDQYDVQQIDAZNb3Njb3cxDzANBgNVBAcMBk1vc2NvdzEQMA4GA1UE
CgwHSVFuZXJneTEUMBIGA1UEAwwLaXFuZXJneS5jb20wHhcNMTkwOTEwMTQ0NzQ4
WhcNMjkwOTA3MTQ0NzQ4WjBXMQswCQYDVQQGEwJSVTEPMA0GA1UECAwGTW9zY293
MQ8wDQYDVQQHDAZNb3Njb3cxEDAOBgNVBAoMB0lRbmVyZ3kxFDASBgNVBAMMC2lx
bmVyZ3kuY29tMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAtCCtLgF8
7IfwEv4kiEdvashHfBe4orpX/ndCp60894yEjjyrbPw/8WGAHyWKPAZ4DK/5/Wi6
ZeO/KFvDs0qP1dYNEzsG8W2NahD0xGB8qSmRWXnmYzqSZkF4cSY3zknjfZiiRXmF
qfUbZtPQNwdvirPpWSPr0SRuPknlS76DwDMEoIGJVZRMH5N9plnHgQDFXw08EloG
tjYjN10q2JF453ZgP70WgbL7dvkwFdqpR6HUdjGnzFvJZ9F6sDurbhGjcqd7RBqk
DqPmgrxgWu0l+s+FScnZ81BTlthNvBcKIc4aKO4FROXXBK31O8WKWHMyIAiU5Ynz
GHVFmTsLUWSSpwIDAQABo1AwTjAdBgNVHQ4EFgQURoc3VosyGDvPpyRFY2ep/mD4
7kAwHwYDVR0jBBgwFoAURoc3VosyGDvPpyRFY2ep/mD47kAwDAYDVR0TBAUwAwEB
/zANBgkqhkiG9w0BAQsFAAOCAQEAWEzEnf3aNSUHoCZxl9y9pQtCJuy4UAvcDdD1
lCzaysO+AkVNfHGczwWdZMwGwRjC8Ao792Ghrm4pW8z2FuBVeEXViMgoyro1ngQ2
KUDyxdUn96w04zo96PPprwJmScOvG3lHDUXt94j7UmWeNeQ4KGlnWUvj1hZqX2tV
DCUBZPMW80dGjgRmDFsz9dDbt3plggsL61o5KKvqeXfL0vV3uIy95y+DnYCV0B1v
ISH8tLRenu8xTITvT8WoceCuf/RutREJmRl8JJ50LZVrARQOFPoxjLRgynvyLrYh
rWnTEAwQKgUNlLmtf4xHSPnn/a2KeAfADt/XuUZivZ6GBTpi5w==
-----END CERTIFICATE-----
'

IQ_CERTIFICATES["mqtt"]='
-----BEGIN CERTIFICATE-----
MIID0TCCArmgAwIBAgIUYh2y22nmGf5SRzgbNd2uri65LVswDQYJKoZIhvcNAQEL
BQAweDELMAkGA1UEBhMCUlUxDzANBgNVBAgMBk1vc2NvdzEPMA0GA1UEBwwGTW9z
Y293MRAwDgYDVQQKDAdJUW5lcmd5MRQwEgYDVQQDDAtpcW5lcmd5LmNvbTEfMB0G
CSqGSIb3DQEJARYQaW5mb0BpcW5lcmd5LmNvbTAeFw0yNDA1MzEwMDIwMjFaFw0z
NDA1MjkwMDIwMjFaMHgxCzAJBgNVBAYTAlJVMQ8wDQYDVQQIDAZNb3Njb3cxDzAN
BgNVBAcMBk1vc2NvdzEQMA4GA1UECgwHSVFuZXJneTEUMBIGA1UEAwwLaXFuZXJn
eS5jb20xHzAdBgkqhkiG9w0BCQEWEGluZm9AaXFuZXJneS5jb20wggEiMA0GCSqG
SIb3DQEBAQUAA4IBDwAwggEKAoIBAQDJjbCzT5H3UBd80dW8rQycFUaHWkmLaWyR
qGCmGb9C0D6KXk2O86fj22EDaGBqNYsi0GAOF2tLvr1amabfW7DOXgnAfyHOPOD6
ruoe4m7Q9BBsqhOuZRiyEO6ud7TO9GbDZ0tygpVwDhOmvv5EqmlmnyRgSirwJrte
/nN+OMpGRuip0enKEAibKtU9RSJU0RDbZ045ecw6x6iVvKM5kcDpDvMqjJWVNY9n
OwrDxOVshLYhC4DyQh7QCmajQ2mTr1jJMggYvZNnoGA99o6FcPl9x7wupgay7ely
FgtVzXbPD78F7qo9vrCz9BCavvVRCmLqh0aA1sNvv+WXAP4pNgNVAgMBAAGjUzBR
MB0GA1UdDgQWBBSlhoXBhmtjPiz/ts8emTxR9a8x9TAfBgNVHSMEGDAWgBSlhoXB
hmtjPiz/ts8emTxR9a8x9TAPBgNVHRMBAf8EBTADAQH/MA0GCSqGSIb3DQEBCwUA
A4IBAQA7A3CMvlkSdbCqZqm8yZoOTM4NZG+uH3YD10GNDPPcNi5ywBjih9XBN9wr
H41cVJrcECzW68mG7HVV6tGxKu0OsenYPF52u8+8wbHXBFSczne/Goga8jH6kFgQ
U8Q5BSu6d58E9dPEI9r429asrziSZ2CLeHEpzSzhTU/VQ0XH5CKpq0R/qQZbozNK
khkXsR76JfILwVbuyKTV+oPDrm8hhQ/sup0eC6qzI26a1ghK11DF3ny3mCJnZKQ6
iYEn1tapKZO6+VNfh9IqSAQFrqoMx/+JWrwYkfWWIqCqTj8tPXOlFBXhNxJTt0R9
bxTwJShQwRVMzqozIXkgyxSiKn0g
-----END CERTIFICATE-----
'

if [[ -v IQ_CERTIFICATES[$NODE_TYPE] ]]; then
    CURR_TIME=$(date +%s)
    ID_CERT="ca_${NODE_TYPE}_${CURR_TIME}.crt"
    ROOT_CERT_PATH="${CA_CERTIFICATES}/${ID_CERT}"
    echo "${IQ_CERTIFICATES[$NODE_TYPE]}" > "${ROOT_CERT_PATH}" || exit 3
    chmod 644 "${ROOT_CERT_PATH}" || exit 4
    cp $SCRIPTPATH/mosquitto.conf /etc/mosquitto/mosquitto.conf  || exit 1

    if [ ! -z "$ADD_BRIDGE" ]; then
        old_bridge=$(mktemp -q)
        cat $BRIDGE_CFG > $old_bridge
        echo -e "$(eval "echo -e \"`cat \"$SCRIPTPATH/bridge.conf.template\"`\"")" > $BRIDGE_CFG || exit 2
        echo '' >> $BRIDGE_CFG
        cat $old_bridge >> $BRIDGE_CFG
    else
        echo -e "$(eval "echo -e \"`cat \"$SCRIPTPATH/bridge.conf.template\"`\"")" > $BRIDGE_CFG || exit 2
    fi

    ######################################
    # Блокируем доступ к MQTT снаружи и ставим пароль на web интерфейс

    echo '
listener 1883 127.0.0.1
allow_anonymous true
acl_file /etc/mosquitto/acl/default.conf
password_file /etc/mosquitto/passwd/default.conf

listener 18883 127.0.0.1
protocol websockets
allow_anonymous true
acl_file /etc/mosquitto/acl/default.conf
password_file /etc/mosquitto/passwd/default.conf
    ' > $LISTENERS_CFG

    ln -s /etc/nginx/sites-available/global_auth /etc/nginx/sites-enabled
    printf "root:$(echo "$(cat /etc/hostname)@$(cat /var/lib/wirenboard/short_sn.conf)🤯" | openssl passwd -apr1 -stdin)\n" > /etc/nginx/passwd
    systemctl restart nginx
    ######################################
    ######################################

    mkdir -p /root/.ssh/
    cp "$SCRIPTPATH/known_hosts" /root/.ssh/  || exit 5

    hostname "$NODE_ID"
    echo "$NODE_ID" > /etc/hostname
else
    echo "Нет сертификата для ${NODE_ID}!" || exit 6
fi

