#!/bin/bash
SCRIPT_PATH="$0"
WORK_DIR=$(dirname "$SCRIPT_PATH")
IQ_PACKAGES_OS=bullseye
BATCH_NO=$(timeout 5 mosquitto_sub -t "/devices/system/controls/Batch No" -C 1)
SHORT_SN=$(timeout 5 mosquitto_sub -t "/devices/system/controls/Short SN" -C 1)

. /etc/os-release

if [ "$VERSION_CODENAME" != "$IQ_PACKAGES_OS" ]; then
    echo "Error: The OS version ($VERSION_CODENAME) does not match the expected version ($IQ_PACKAGES_OS)"
    exit 1
fi

source "$WORK_DIR/nmhelper/nmsetup.sh"

#default
LINE_A=rs485
LINE_B=rs485
MODEM=wb_4g
TEMP_SSID=
TEMP_PASSWORD=

help () {
    echo
    echo "To install the controller, set the parameters:"
    echo "--no_migrate          Only upgrade mosquitto template. With this parameter, the following arguments will be ignored!!!"
    echo "-m, --move            Only move controller to new server. With this parameter, the following arguments will be ignored!!!"
    echo "-t, --type            TYPE       [ dev | stage | online ]"
    echo "-i, --id              NODE_ID    Some controller id without spaces."
    echo "-u, --mqtt_username   USERNAME   Username on MQTT Server."
    echo "-p, --mqtt_password   PASSWORD   Password on MQTT Server."
    echo "--line_a              PROTOCOL   First line protocol [ rs485 ]."
    echo "--line_b              PROTOCOL   First line protocol [ rs485 | uart ]."
    echo
    echo "-z, --zigbee          Use to support ZigBee."
    echo "--modem               TYPE [ wb_4g | wb_4g ]   Use to 4G modem."
    echo "-o, --offline_install DIRECTORY     Use to install IQ packages from local directory."
    echo "-r, --root_password   PASSWORD      Set custom root password."
    echo "--use_wifi            SSID PASSWORD     Use Wi-Fi connection with provided SSID and PASSWORD."
    echo "--beta                Use BETA Debian-packages."
    echo
}


while [ -n "$1" ]
do
case "$1" in
-t|--type)
    NODE_TYPE="$2"
    shift ;;

-i|--id)
    NODE_ID="$2"
    shift ;;

-u|--mqtt_username)
    LOGIN="$2"
    shift ;;

-p|--mqtt_password)
    PASSWORD=$2
    shift ;;

-o|--offline_install)
    DEB_PACKAGES_DIR=$2
    shift ;;

--line_a)
    LINE_A=$2
    shift ;;

--line_b)
    LINE_B=$2
    shift ;;

-w|--zwave)
    ZWAVE=true ;;

-b|--zigbee)
    ZIGBEE=true ;;

-m|--move)
    ONLY_MOVE=true ;;

--modem)
    MODEM=$2
    shift ;;

-r|--root_password)
    ROOT_PWD="$2"
    shift ;;

--use_wifi)
    TEMP_SSID="$2"
    TEMP_PASSWORD="$3"
    shift 2 ;;

--beta)
    BETA=true ;;

*) help ;;
esac
shift
done

TIMESTAMP=$(date +"%s")

if [ "$ONLY_MOVE" = true ]; then
    ZWAVE=
    ZIGBEE=
    DEB_PACKAGES_DIR=

    if [ -z "$NODE_ID" ]; then
        NODE_ID=$(cat /etc/hostname)
    fi
fi

OLD_NODE_ID=$(cat /etc/hostname)

if [ -z "$NODE_TYPE" ] || \
    [ -z "$NODE_ID" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ]; then
    help
    exit 1
fi


#######################################
# Настройка конфигурации
if [ -z "$ONLY_MOVE" ]; then
    $WORK_DIR/configure_hardware.py -z wb_cc2530 -m ${MODEM} -a ${LINE_A} -b ${LINE_B} || exit 5
    service wb-hwconf-manager restart
    echo "wb-hwconf-manager configured"
fi
#######################################
#######################################


#######################################
# Настройка сети
if [ -z "$ONLY_MOVE" ]; then
    update_priority high wb-eth0
    update_priority low wb-gsm-sim1
    nmcli con mod wb-gsm-sim1 connection.autoconnect yes
    update_priority low wb-gsm-sim2
    nmcli con mod wb-gsm-sim2 connection.autoconnect yes

    if [ -n "$TEMP_SSID" ] && [ -n "$TEMP_PASSWORD" ]; then
        upsert_wifi_connection "medium" "temp_access_point" "$TEMP_SSID" "$TEMP_PASSWORD"
    fi

    upsert_wifi_connection "high" "ServiceAP" "service_iqnergy_ap" "PA_YGRENQI_ECIVRES"

    # upsert_ethernet_connection "high" "eth0"
    # upsert_gsm_connection "low" "AutoGSM"

    service wb-connection-manager restart
    sleep 5
    wait_for_network 120 iqnergy.com || exit 1
fi
#######################################
#######################################

#######################################
# Установка недостающих пакетов и удаление лишних
if [ -z "$ONLY_MOVE" ]; then
apt-get update

apt-get install -y \
    apt-listchanges \
    rsync \
    wb-mcu-fw-updater \
    socat \
    python3 \
    python3-pip \
    apt-transport-https || exit 3
    pip3 install paho-mqtt pyserial pyinotify || exit 4
fi
#######################################
#######################################

# Установка ПО IQnergy
if [ -z "$ONLY_MOVE" ]; then

    if [ -z "$BETA" ]; then
        echo 'deb https://deb.iqnergy.com/debian/ bullseye main' > /etc/apt/sources.list.d/iqnergy.list
    else
        echo 'deb https://deb.iqnergy.com/debian/ bullseye beta' > /etc/apt/sources.list.d/iqnergy.list
    fi

    wget -O - http://deb.iqnergy.com/signature.gpg.key | apt-key add - || exit 8
    systemctl restart apt-daily.timer || exit 9
    apt-get -y update

    if [ -z "$DEB_PACKAGES_DIR" ]; then
        export DEBIAN_FRONTEND=noninteractive ; apt-get -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" install -y iq-iqnergy-software-installer || exit 10
    else
        export DEBIAN_FRONTEND=noninteractive ; apt-get -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" install -y "$DEB_PACKAGES_DIR"*deb -y || exit 11
    fi

    # конфигурируем mosquitto
    "$WORK_DIR"/configure_mosquitto.sh --mqtt_username ${LOGIN} --mqtt_password ${PASSWORD} --type ${NODE_TYPE} --id ${NODE_ID} || exit 6

    systemctl restart mosquitto || exit 7

fi
#######################################
#######################################


#######################################
# Установка ZigBee
if [ "$ZIGBEE" = true ]; then
    ZIGBEE_DIR=/mnt/data/root/zigbee2mqtt
    apt install -y zigbee2mqtt wb-zigbee2mqtt

    CH=`expr ${NODE_ID:2} % 4`

    case $CH in

      0)
        ZIGBEE_CHANNEL=11
        ;;

      1)
        ZIGBEE_CHANNEL=15
        ;;

      2)
        ZIGBEE_CHANNEL=20
        ;;

      *)
        ZIGBEE_CHANNEL=25
        ;;
    esac


    ZIGBEE_PANID=`expr ${NODE_ID:2} % 16383`
    DEFAULT_ZIGBEE_KEY='DEFAULT!ZigbeeKey'
    ANSII_KEY=`echo $NODE_ID$NODE_ID$DEFAULT_ZIGBEE_KEY | cut -c1-16`
    ANSII_EXT_PANID=`echo $NODE_ID$NODE_ID$DEFAULT_ZIGBEE_KEY | cut -c1-8`
    ZIGBEE_NETWORK_KEY=`printf "$ANSII_KEY" | xxd -p | sed 's/\(..\)/0x&, /g; s/, $//;'`
    ZIGBEE_EXT_PANID=`printf "$ANSII_EXT_PANID" | xxd -p | sed 's/\(..\)/0x&, /g; s/, $//;'`

    echo -e "$(eval "echo -e \"`cat \"$WORK_DIR/zigbee/configuration.yaml.template\"`\"")" | tee "$ZIGBEE_DIR/data/configuration.yaml" || exit 12
    cp "$WORK_DIR/zigbee/iqnergy.js" "$ZIGBEE_DIR/node_modules/zigbee-herdsman-converters/devices/"
fi
#######################################
#######################################


#######################################
# Меняем пароль по-умолчанию на свой
if [ -z "$ONLY_MOVE" ]; then
    echo -e "\nCHANGING ROOT PASSWORD AND ADDING \"$USER_NAME\" USER"
    echo "New password for ROOT"
    echo -e "$ROOT_PWD\n$ROOT_PWD" | passwd || exit 13
fi
#######################################
#######################################


######################################
# Запоминаем время
if [ -z "$ONLY_MOVE" ]; then
    hwclock --systohc --localtime
fi
######################################
######################################

apt-get clean

clear
cat "$WORK_DIR/IQ_logo.txt"
sync

YELLOW='\033[1;33m'
GREEN='\033[1;32m'
NC='\033[0m'

printf "Old ID: ${YELLOW}$OLD_NODE_ID${NC}\n"
printf "New ID: ${YELLOW}$(cat /etc/hostname)${NC}\nServer: ${YELLOW}$NODE_TYPE${NC}\n"
printf "Revision: ${YELLOW}${BATCH_NO}${NC}\n"
printf "SN: ${YELLOW}${SHORT_SN}${NC}\n"
printf "${GREEN}Install done! ${NC}\n"
echo
printf "${YELLOW}Removing access point and reboot...${NC}\n"
remove_connection "temp_access_point" >> /dev/null
remove_connection "wb-ap" >> /dev/null && \
printf "${YELLOW}Rebooting...${NC}\n" && \
sync; sync; sleep 1 && reboot
