#!/bin/bash

SELFNAME="$0"
SELFDIR=$(dirname "$SELFNAME")

#default
LINE_A=rs485
LINE_B=rs485
MODEM=wb_4g

help () {
    echo
    echo "To install the controller, set the parameters:"
    echo "--no_migrate          Only upgrade mosquitto template. With this parameter, the following arguments will be ignored!!!"
    echo "-m, --move            Only move controller to new server. With this parameter, the following arguments will be ignored!!!"
    echo "-t, --type            TYPE       [ dev | stage | online ]"
    echo "-i, --id              NODE_ID    Some controller id without spaces."
    echo "-u, --mqtt_username   USERNAME   Username on MQTT Server."
    echo "-p, --mqtt_password   PASSWORD   Password on MQTT Server."
    echo "line_a                PROTOCOL   First line protocol [ rs485 ]."
    echo "line_b                PROTOCOL   First line protocol [ rs485 | uart ]."
    echo
    echo "-z, --zigbee          Use to support ZigBee."
    echo "--modem               TYPE [ wb_4g | wb_4g ]   Use to 4G modem."
    echo "-o, --offline_install DIRECTORY     Use to install IQ packages from local directory."
    echo "-r, --root_password   PASSWORD      Set custom root password."
    echo "--beta                Use BETA Debian-packages."
    echo
}

while [ -n "$1" ]
do
case "$1" in
-t|--type)
    NODE_TYPE="$2"
    shift ;;

-i|--id)
    NODE_ID="$2"
    shift ;;

-u|--mqtt_username)
    LOGIN="$2"
    shift ;;

-p|--mqtt_password)
    PASSWORD=$2
    shift ;;

-o|--offline_install)
    DEB_PACKAGES_DIR=$2
    shift ;;

--line_a)
    LINE_A=$2
    shift ;;

--line_b)
    LINE_B=$2
    shift ;;

--board)
    BOARD_CONF=$2
    shift ;;

-w|--zwave)
    ZWAVE=true ;;

-z|--zigbee)
    ZIGBEE=true ;;

--modem)
    MODEM=$2 ;;

-r|--root_password)
    ROOT_PWD="$2"
    shift ;;

--beta)
    BETA=true ;;

*) help ;;
esac
shift
done

if [ -z "$NODE_ID" ]; then
    NODE_ID=$(cat /etc/hostname)
fi

if [ -z "$NODE_TYPE" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ]; then
    help
    exit 1
fi

apt-get update

#######################################
# Установка недостающих пакетов и удаление лишних
if [ -z "$ONLY_MOVE" ]; then
apt-get install -y \
    apt-listchanges \
    rsync \
    wb-mcu-fw-updater \
    resolvconf \
    socat \
    python3 \
    python3-pip \
    apt-transport-https \
    nodejs || exit 2
    pip3 install paho-mqtt && pip3 install pyserial || exit 3
fi
#######################################
#######################################


#######################################
# Установка ПО IQnergy
if [ -z "$ONLY_MOVE" ]; then
    if [ -z "$DEB_PACKAGES_DIR" ]; then

        if [ -z "$BETA" ]; then
            echo 'deb https://deb.iqnergy.com/debian/ stretch main' > /etc/apt/sources.list.d/iqnergy.list
        else
            echo 'deb https://deb.iqnergy.com/debian/ stretch beta' > /etc/apt/sources.list.d/iqnergy.list
        fi

        wget -O - http://deb.iqnergy.com/signature.gpg.key | apt-key add - || exit 4
        systemctl restart apt-daily.timer || exit 5
        apt-get -y update
        apt-get -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" install -y iq-iqnergy-software-installer || exit 6
    else
        apt-get install -y "$DEB_PACKAGES_DIR"/iq*deb
    fi
fi

#######################################
#######################################


#######################################
# Настройка сетевых адаптеров
if [ -z "$ONLY_MOVE" ]; then
    $SELFDIR/configure_hardware.py -z wb_cc2530 -m ${MODEM} -a ${LINE_A} -b ${LINE_B} || exit 7
fi
#######################################
#######################################


#######################################
# Активируем правила monit
ln -f -s /etc/monit/conf-available/wb-rules /etc/monit/conf-enabled/wb-rules
ln -f -s /etc/monit/conf-available/wb-rules-lastseen /etc/monit/conf-enabled/wb-rules-lastseen
ln -f -s /etc/monit/conf-available/data-handler /etc/monit/conf-enabled/data-handler
systemctl restart monit
#######################################
#######################################


#######################################
# Конфигурирование lsyncd
mkdir -p /etc/lsyncd/
mkdir -p /var/log/lsyncd/

echo -e "$(eval "echo -e \"`cat \"$SELFDIR/lsyncd/lsyncd.conf.lua.template\"`\"")" > "/etc/lsyncd/lsyncd.conf.lua" || exit 8
rsync "$SELFDIR/lsyncd/lsyncd" "/etc/logrotate.d/" || exit 9

if [ -z "$ONLY_MOVE" ]; then
    rm /run/systemd/generator.late/lsyncd.service
    rm /etc/init.d/lsyncd
    cp $SELFDIR/lsyncd/lsyncd.service /lib/systemd/system/
    systemctl daemon-reload && systemctl enable lsyncd
fi

echo -e \
'/var/log/lsyncd/lsyncd.log {
  dateext
  rotate 1
  missingok
  size 200k
}' > /etc/logrotate.d/lsyncd
chmod 644 /etc/logrotate.d/lsyncd
rm -f /var/log/lsyncd/lsyncd.log.*
#######################################
#######################################


#######################################
# Меняем пароль по-умолчанию на свой
if [ -z "$ONLY_MOVE" ]; then
    echo -e "\nCHANGING ROOT PASSWORD AND ADDING \"$USER_NAME\" USER"
    echo "New password for ROOT"
    echo -e "$ROOT_PWD\n$ROOT_PWD" | passwd || exit 10
fi
#######################################
#######################################


#######################################
# Немного меняем приветствие
sed -i -e 's/figlet wirenboard/figlet IQnergy/g' /etc/update-motd.d/10-wirenboard
#######################################
#######################################

#######################################
# Очищаем мусор
apt-get clean
rm /tmp/iq*deb
#######################################
#######################################


cat "$SELFDIR/IQ_logo.txt"
sync

YELLOW='\033[1;33m'
GREEN='\033[1;32m'
NC='\033[0m'

printf "${YELLOW}$(cat /etc/hostname): server: $NODE_TYPE"

if [ -n "$FOURTH_GENERATION_MODEM" ]; then
    printf "; 4G"
fi

if [ -n "$ZIGBEE" ]; then
    printf "; ZigBee"
fi

printf "\n${GREEN}Install done! ${NC}\n"
