#!/bin/bash

SCRIPT_DIR=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )
NEW_TIMESTAMP=
NODE_ID=$(cat /etc/hostname)
BRIDGE_CFG=/mnt/data/etc/mosquitto/conf.d/20bridge.conf
LISTENERS_CFG=/mnt/data/etc/mosquitto/conf.d/10listeners.conf
MQTT_CFG=/etc/mosquitto/mosquitto.conf

mosquitto_pub -q 2 -r -t /iq/bridge/migrate/status -m \
    "$(date) Готово к миграции"

# Ждем команды для миграции
mosquitto_sub -t /iq/bridge/migrate -R -C 5

# Запрвшиваем аргументы
mosquitto_pub -q 2 -r -t /iq/bridge/migrate/status -m \
    "$(date) Начинаем процесс обновления. Отправьте аргументы в топик /iq/{NODE_ID}/migrate/args"

# Ждем аргументы
ARGS=$(timeout 160 mosquitto_sub -t /iq/bridge/migrate/args -C 1 -R)
if [ -z "$ARGS" ]; then
    mosquitto_pub -q 2 -r -t /iq/bridge/migrate/status -m "$(date) Аргументы не были получены. Начните сначала."
    exit 1
fi

# Аргументы получены, продолжаем...
mosquitto_pub -q 2 -r -t /iq/bridge/migrate/status -m \
    "$(date) Аргументы получены, продолжаем"

# Делаем бекап текущего конфига mosquitto. Если не получается, выходим с ошибкой
cp $MQTT_CFG $MQTT_CFG.backup && cp $BRIDGE_CFG $BRIDGE_CFG.backup && cp $LISTENERS_CFG $LISTENERS_CFG.backup
BACKUP=$?

if [ $BACKUP -ne 0 ] ; then
    mosquitto_pub -q 2 -t /iq/bridge/migrate/status -m "$(date) Ошибка создания резервной копии старого конфига"
    exit 3
fi

# Выполняем скрипт обновления
/usr/bin/iqnergy/installer/configure_mosquitto.sh ${ARGS}
RET=$?
if [ $RET -ne 0 ] ; then
    mosquitto_pub -q 2 -t /iq/bridge/migrate/status -m "$(date) Ошибка выполнения скрипта обновления. $RET"
else
    systemctl restart mosquitto
    RET=$?
fi

# Если mosquitto перезапустился и конфиг обновился без ошибок, ждем пинг от сервера
if [ $RET -eq 0 ]; then
    sleep 10
    NEW_TIMESTAMP=$(timeout 45 mosquitto_sub -t /devices/lastseen/controls/server/on -R -C 3)
else
    mosquitto_pub -r -t /iq/bridge/migrate/status -m "$(date) Ошибка выполнения миграции. Код $INSTALLER_RET"
fi

# Если timestamp пустой, восстанавливаемся из бекапа
# Пустой timestamp будет в случае ошибки обновления или ошибки перезапуска mosquitto
if [ -z "$NEW_TIMESTAMP" ]; then
    cp $MQTT_CFG.backup $MQTT_CFG
    cp $BRIDGE_CFG.backup $BRIDGE_CFG
    cp $LISTENERS_CFG.backup $LISTENERS_CFG
    systemctl restart mosquitto
    sleep 10
    mosquitto_pub -q 2 -r -t /iq/bridge/migrate/status -m "$(date) Timestamp не получен! Выполнен откат к старым настройкам."
    exit 4
fi

mosquitto_pub -q 2 -r -t /iq/bridge/migrate/status -m "$(date) Шаблон mosquitto обновлен!"

#######################################
# Генерация ключей, для удаленного доступа с сервера по ssh и логирования

mkdir -p /root/.ssh/
cp "/usr/bin/iqnergy/installer/known_hosts" /root/.ssh/

pub_id_hostname=$(echo "$( [ -e ~/.ssh/id_rsa.pub ] && cat ~/.ssh/id_rsa.pub )" | awk '{split($NF,a,"@");print a[2]}');

if [ "$(cat /etc/hostname)" != "$pub_id_hostname" ]; then
    rm -f /root/.ssh/id_rsa*
    ssh-keygen -f /root/.ssh/id_rsa -N ""
fi

# Отправляем ключ на новый сервер MQTT
PUBKEY=$(echo -e "Публичный ключ на $(date):\n$(cat /root/.ssh/id_rsa.pub)")
mosquitto_pub -t /iq/bridge/pubkey -r -m "$PUBKEY" -q 2

#######################################
#######################################

#######################################
# Обновляем конфиг lsyncd
${SCRIPT_DIR}/configure_lsyncd.sh
mosquitto_pub -q 2 -r -t /iq/bridge/migrate/status -m "$(date) Шаблон lsyncd обновлен!"
#######################################
#######################################


#######################################
# Обновляем информацию о версии шаблона mosquitto
mosquitto_pub -r -t /iq/bridge/upgrade/bridge_version -q 2 -m "$(head -n 2 /mnt/data/etc/mosquitto/conf.d/20bridge.conf)"
mosquitto_pub -r -t /devices/system/controls/bridges -q 2 -m "$(grep -a address /mnt/data/etc/mosquitto/conf.d/20bridge.conf)"
#######################################
#######################################
