#!/usr/bin/env python3

import subprocess
import os

class NMSetupWrapper:
    def __init__(self):
        self.script_path = os.path.join(os.path.dirname(__file__), 'nmsetup.sh')

    def call_function(self, function_name, *args):
        cmd = ['bash', self.script_path, function_name] + list(args)
        try:
            subprocess.run(cmd, check=True)
        except subprocess.CalledProcessError as e:
            print(f"Error executing {function_name}: {e}")
            return False
        return True

    def upsert_wifi_connection(self, priority, con_name, ssid, password):
        return self.call_function("upsert_wifi_connection", priority, con_name, ssid, password)

    def upsert_ethernet_connection(self, priority, con_name):
        return self.call_function("upsert_ethernet_connection", priority, con_name)

    def upsert_gsm_connection(self, priority, con_name):
        return self.call_function("upsert_gsm_connection", priority, con_name)

    def remove_connection(self, con_name):
        return self.call_function("remove_connection", con_name)

    def wait_for_network(self, timeout_in_seconds, node_address):
        return self.call_function("wait_for_network", timeout_in_seconds, node_address)

    def update_priority(self, priority, con_name):
        return self.call_function("update_priority", priority, con_name)
