#!/usr/bin/env python3
import json
import sys
import os

def update_priority(con_name, action, configuration, priority=None):
    default_json = {
        "connectivity_check_url": "http://network-test.debian.org/nm",
        "tiers": {"high": [], "low": [], "medium": []}
    }

    # Загрузка или инициализация JSON конфигурации
    if os.path.isfile(configuration):
        with open(configuration, 'r') as file:
            try:
                config_data = json.load(file)
                # Проверяем, что config_data является словарём
                if not isinstance(config_data, dict):
                    raise ValueError("Config data is not a dictionary")
            except (json.JSONDecodeError, ValueError):
                print(f"Configuration file '{configuration}' contains invalid JSON. Reinitializing...")
                config_data = default_json
    else:
        print(f"Configuration file '{configuration}' does not exist. Creating it...")
        config_data = default_json

    # Обновление JSON, чтобы убедиться, что все ключи присутствуют
    config_data = {**default_json, **config_data}
    for tier in default_json['tiers']:
        config_data['tiers'].setdefault(tier, [])

    config_data.pop('connections', None)

    # Удаление соединения из всех уровней приоритета
    for tier in config_data['tiers']:
        config_data['tiers'][tier] = [cn for cn in config_data['tiers'][tier] if cn != con_name]

    # Добавление соединения в указанный уровень приоритета, если действие не 'remove'
    if action != 'remove':
        if priority in config_data['tiers']:
            config_data['tiers'][priority].append(con_name)
        else:
            print(f"Invalid priority level '{priority}'. Valid levels are: 'high', 'medium', 'low'.")
            return

    # Сохранение обновлённой конфигурации
    with open(configuration, 'w') as file:
        json.dump(config_data, file, indent=4)

    # Вывод содержимого обновлённого файла
    print(json.dumps(config_data, indent=4))

if __name__ == "__main__":
    if len(sys.argv) < 4 or len(sys.argv) > 5:
        print("Usage: update_priority.py <con_name> <action> <configuration> [<priority>]")
        sys.exit(1)

    priority = sys.argv[4] if len(sys.argv) == 5 else None

    update_priority(sys.argv[1], sys.argv[2], sys.argv[3], priority)
