const extend = require('../lib/extend');
const exposes = require('../lib/exposes');
const fz = require('../converters/fromZigbee');
const tz = require('../converters/toZigbee');
const reporting = require('../lib/reporting');
const e = exposes.presets;
const ea = exposes.access;


const iqnergy = {
    fz: {
        relay_status: {
            cluster: 'genOnOff',
            type: ['attributeReport', 'readResponse'],
            convert: (model, msg, publish, options, meta) => {

                return {
                    K1: msg.data['onOff'],
                    K2: msg.data['1'],
                    K3: msg.data['2']
                };
            },
        },
        energy: {
            cluster: 'haElectricalMeasurement',
            type: ['attributeReport', 'readResponse'],
            convert: (model, msg, publish, options, meta) => {

                var raw_energy_a = msg.data['2816'];
                var raw_energy_b = msg.data['2817'];
                var raw_energy_c = msg.data['2818'];

                if (((typeof raw_energy_a === 'string' || raw_energy_a instanceof String))
                 && ((typeof raw_energy_b === 'string' || raw_energy_b instanceof String))
                 && ((typeof raw_energy_c === 'string' || raw_energy_c instanceof String))) {
                    return {
                        'energy': parseInt(raw_energy_a.replace(/0x/g, ''), 16) / 1000000,
                        'energy_phase_b': parseInt(raw_energy_b.replace(/0x/g, ''), 16) / 1000000,
                        'energy_phase_c': parseInt(raw_energy_c.replace(/0x/g, ''), 16) / 1000000
                    };
                }

                return {};
            },
        },
    },
};

module.exports = [
    {
        zigbeeModel: ['EMRz02'],
        model: 'EMRz',
        vendor: 'IQnergy energy meter & control',
        description: 'EMRz v.2 Load measurement and control',
        fromZigbee: [fz.temperature,
                     fz.electrical_measurement,
                     iqnergy.fz.relay_status,
                     iqnergy.fz.energy],

        toZigbee: [tz.light_onoff_brightness],
        exposes: [exposes.numeric('power', ea.STATE).withUnit('W').withDescription('Instantaneous measured power on phase A'),
                  exposes.numeric('power_phase_b', ea.STATE).withUnit('W').withDescription('Instantaneous measured power on phase B'),
                  exposes.numeric('power_phase_c', ea.STATE).withUnit('W').withDescription('Instantaneous measured power on phase C'),
                  e.current(),
                  e.current_phase_b(),
                  e.current_phase_c(),
                  e.temperature()],
        configure: async (device, coordinatorEndpoint, logger) => {
            const endpoint = device.getEndpoint(1);
            await reporting.bind(endpoint, coordinatorEndpoint, ['haElectricalMeasurement']);
            await endpoint.read('haElectricalMeasurement', ['acVoltageMultiplier', 'acVoltageDivisor', 'acCurrentMultiplier']);
            await endpoint.read('haElectricalMeasurement', ['acCurrentDivisor', 'acPowerMultiplier', 'acPowerDivisor']);
        },
    },
];
